@@ -1,34 +1,33 @@
 import numbers
+from contextlib import suppress
+import copy
 from dataclasses import asdict, is_dataclass
 
 
 class Laozi:
     @staticmethod
-    def parse_final(obj):
+    def parse_final(key, obj):  # noqa
         return obj
 
     @classmethod
-    def parse_str(cls, obj, prefix='', ctx={}):
-        obj = cls.parse_final(obj)
+    def parse_str(cls, key, obj, prefix='', ctx={}):
+        obj = cls.parse_final(key, obj)
         if not prefix:
             yield f'"{obj}"'
         else:
             yield f'{prefix[:-1]}="{obj}"'
 
     @classmethod
-    def parse_int(cls, obj, prefix='', ctx={}):
-        obj = cls.parse_final(obj)
+    def parse_int(cls, key, obj, prefix='', ctx={}):
+        obj = cls.parse_final(key, obj)
         yield f'{prefix[:-1]}={obj}'
 
     @classmethod
-    def parse_set(cls, obj, prefix='', ctx={}):
-        yield from cls.parse_list(obj, prefix, ctx=ctx)
+    def parse_set(cls, key, obj, prefix='', ctx={}):
+        yield from cls.parse_list(key, obj, prefix, ctx=ctx)
 
     @classmethod
-    def parse_dict(cls, obj, prefix='', ctx={}):
-        print(ctx)
-        print(id(obj))
-
+    def parse_dict(cls, key, obj, prefix='', ctx={}):
         ctx['parsed_ids'].append(id(obj))
         if not hasattr(obj, 'items') and hasattr(obj, '__dict__'):
             obj = obj.__dict__
@@ -36,18 +35,18 @@ class Laozi:
             if id(value) in ctx['parsed_ids']:
                 yield f'{prefix}{key}=...'
                 return
-            yield from cls.get_parser_for(value)(value,
-                                                 prefix=f'{prefix}{key}.',
-                                                 ctx=ctx)
+            parser = cls.get_parser_for(value)
+            yield from parser(key, value, prefix=f'{prefix}{key}.', ctx=ctx)
 
     @classmethod
-    def parse_dataclass(cls, obj, prefix='', ctx={}):
-        yield from cls.parse_dict(asdict(obj), prefix, ctx=ctx)
+    def parse_dataclass(cls, key, obj, prefix='', ctx={}):
+        yield from cls.parse_dict(key, asdict(obj), prefix, ctx=ctx)
 
     @classmethod
-    def parse_list(cls, obj, prefix='', ctx={}):
+    def parse_list(cls, key, obj, prefix='', ctx={}):
         for key, value in enumerate(obj):
-            yield from cls.get_parser_for(value)(value,
+            yield from cls.get_parser_for(value)(key,
+                                                 value,
                                                  prefix=f'{prefix}{key}.',
                                                  ctx=ctx)
 
@@ -65,10 +64,13 @@ class Laozi:
         return getattr(cls, parser, default_parser)
 
     @classmethod
-    def unserializable(cls, obj, prefix='', ctx={}):
+    def unserializable(cls, key, obj, prefix='', ctx={}):
         yield f'{prefix[:-1]}="{obj.__repr__()}"'
 
     @classmethod
     def parse(cls, input_obj):
+        with suppress(Exception):
+            input_obj = copy.deepcopy(input_obj)
         ctx = {'parsed_ids': []}
-        return '; '.join(cls.get_parser_for(input_obj)(input_obj, ctx=ctx))
+        return '; '.join(
+            cls.get_parser_for(input_obj)(None, input_obj, ctx=ctx))
